/* z_base.h (ZRAM: base definitions)
   Ambros Marzetta

   For more information see our WWW page at
   
         http://wwwjn.inf.ethz.ch/ambros/zram.html
	 
 * $Source: /home/schilte/marzetta/work/distr/zram/RCS/z_base.h,v $
 * $Revision: 1.8 $
 * $Date: 1996/06/03 19:00:37 $
 * $Log: z_base.h,v $
 * Revision 1.8  1996/06/03  19:00:37  marzetta
 * New definition of z_NewNode.
 *
 * Revision 1.7  1996/05/08  20:28:09  marzetta
 * Eliminated z_enum.h
 *
 * Revision 1.6  1996/04/09  13:20:21  marzetta
 * new Install procedures
 *
 * Revision 1.5  1995/09/14  12:25:56  marzetta
 * deep copy
 *
 * Revision 1.4  1995/08/22  12:39:12  marzetta
 * New message passing, MPI
 *
 * Revision 1.2  1995/08/02  13:28:46  marzetta
 * Portability to macintosh
 *
 */

#ifndef _Z_BASE_H_
#define _Z_BASE_H_
#include <assert.h>

#ifdef Music
#include "musiclib.h"
#define malloc(x) dmalloc(x, MT_SLOW)
#endif

/* define MINT as 32 bit or 64 bit integer */
#ifdef A7
#include "music.h"
typedef signed MINT int8, int16, int32;
typedef unsigned MINT boolean, uint8, uint16, uint32;
typedef MFLOAT float32;
#else /* A7 */
#if defined(Music) || defined(MusicFE) /* ... */
#include "m_types.h"
typedef signed MINT int8, int16, int32;
typedef unsigned MINT boolean, uint8, uint16, uint32;
typedef MFLOAT float32;
#else /* not Music */
typedef char boolean;
typedef signed char int8;
typedef unsigned char uint8;
typedef signed short int int16;
typedef unsigned short int uint16;
#ifdef macintosh /* int is sometimes 16 bit and sometimes 32 bit */
typedef signed long int int32;
typedef unsigned long int uint32;
#define MINT long int
#else
typedef signed int int32;
typedef unsigned int uint32;
#define MINT int
#endif /* macintosh */
typedef float float32;
#endif /* Music */
#endif /* A7 */
#ifdef __alpha
typedef signed long int int64;
typedef unsigned long int uint64;
#define MAXint64	9223372036854775807 	/* 9E18 */
#endif /* __alpha */

#define MAXint8		127
#define MAXint16	32767
#define MAXint32	2147483647 		/* 2E9 */
#define MAXMINT		2147483647


/* The z_NodeP type ------------------------------------------------------*/

typedef struct z_Node *z_NodeP;

#define Z_NODE_HEADER	uint32 len;  /* len = sizeof(this node) */ \
                        uint16 tag; \
                        uint16 children; /* number of z_NodeP following */ \
                        z_NodeP parent, child, sibling

struct z_Node {
  Z_NODE_HEADER;
};

#define Z_NUM_TAGS 50
/* grep '#define[ 	]*[A-Za-z_]*_TAG[ 	]' *.[hc] */


/* Procedure types ------------------------------------------------------ */

typedef void (*z_VoidProc)(void);

typedef void (*z_PtrProc)(void *);

typedef void (*z_NodeProc)(z_NodeP global, z_NodeP n);

typedef boolean (*z_ConditionProc)(z_NodeP global, z_NodeP n);
#define Z_CONDITION_ALWAYS	(z_ConditionProc) 0

typedef boolean (*z_EqualProc)(z_NodeP global, z_NodeP x, z_NodeP y);
/* Return true if x equals y. Both of them may be NULL. */


/* Interrupts ----------------------------------------------------------- */

extern volatile boolean z_checkpoint;
/* Will be set when some timer expires or when the process receives a
   terminate signal. */

extern volatile boolean z_terminate;
/* Will be set in addition to z_checkpoint when the process receives a
   terminate signal. */


/* Memory management ---------------------------------------------------- */

extern long z_memsize;
/* constant: estimate of size of memory available to this process */

extern long z_memavail;
/* size of memory currently available to this process */
/* Should be modified only by z_NewNodes, z_NewCopyNode and z_FreeNode. */

extern long z_numNodes[Z_NUM_TAGS];
/* Should be modified only by z_NewNodes, z_NewCopyNode and z_FreeNode. */
/* new version: function */

#define z_InstallClass(name, children, reserved1, reserved2) \
        z__InstallClass(#name, children, reserved1, reserved2)

int z__InstallClass(const char *name, int children,
		    int reserved1, int reserved2);
/* Install a node type. z__InstallClass returns a tag to be used in
   subsequent calls to z_NewNode and in accesses to z_numNodes. This is
   a collective operation. reserved1 and reserved2 must be 0. */

z_NodeP z_NewNode(MINT tag, MINT len);
/* Allocate a z_Node, initialize tag, len, children */

z_NodeP z_NewCopyNode(z_NodeP source);
/* Allocate a z_Node identical to source (deep copy) */

void z_FreeNode(z_NodeP node);
/* Deallocate the node and its children (deep free). */

void z_ReSizePointer(void **ptr, long *size, long newsize);
/* Allocate if *size < newsize. Don't copy any data. */


/* Procedures to be supplied by the application ------------------------ */

void initproc (void);
/* This function is called once on every processing element. */

int mmain(int argc, char **argv, char **envp);
/* Application dependent main program. It is called after
   initproc() on the front end. */


#endif /* _Z_BASE_H_ */
