/* z_signal.c (ZRAM: signal handler)
   Ambros Marzetta

 * $Source: /home/schilte/marzetta/work/distr/zram/RCS/z_signal.c,v $
 * $Revision: 1.2 $
 * $Date: 1996/05/15 16:43:58 $
 * $Log: z_signal.c,v $
 * Revision 1.2  1996/05/15  16:43:58  marzetta
 * Moved an #ifdef macintosh
 *
 * Revision 1.1  1996/05/08  20:30:42  marzetta
 * Initial revision
 *
 */

#include "z_base.h"
#include "z_par.h"
#include "z_signal.h"
#include "z_time.h"
#include "stdio.h"
#include <errno.h>
#include <signal.h>
#ifndef macintosh
#include <sys/time.h>
#endif /* macintosh */

boolean z_signal_trace=1;	/* trace the signal handler */
				/* warning: may interrupt any output */
long z_signal_checkpointTime=3600;
                                /* time in seconds between checkpoints */

/* Signal handler ----------------------------------------------------- */

#ifndef macintosh
static void z_term_handler(int signal)
{
  if (z_signal_trace)
    printf("\n[%2d]Got signal %d (time=%f)\n",
	   z_myproc, signal, z_Wtime());
  fflush(stdout);
  /* Is it allowed to call fflush from a signal handler?
     Is fflush reentrant? */
  z_terminate = (signal == SIGTERM);
  z_checkpoint = 1;
} /* z_term_handler */
#endif

void z_StartCheckpointTimer(void)
{
#if !defined(macintosh) && !defined(cenju3)
  struct itimerval new;

  new.it_interval.tv_sec = z_signal_checkpointTime;
  new.it_interval.tv_usec = 0;
  new.it_value.tv_sec = z_signal_checkpointTime;
  new.it_value.tv_usec = 0;
  
  if (setitimer(ITIMER_REAL, &new, NULL))
    perror("setitimer");
#endif
} /* z_StartCheckpointTimer */


/* Initialization ----------------------------------------------------- */

void z_signal_Init(void)
{
  /* Install a signal handler for the terminate signal. When the process
     receives the signal, it sets the z_terminate and z_checkpoint variables.
     */

#if !defined(macintosh) && !defined(cenju3)
  static struct sigaction action, o_action;

  action.sa_handler = z_term_handler;
  /* sigset_t is defined differently depending on the operation system.
     declaring the struct as static simplifies the initialization. */
  
  if (sigaction(SIGALRM, &action, &o_action))
    perror("sigaction");
  if (sigaction(SIGTERM, &action, &o_action))
    perror("sigaction");
#endif
} /* z_signal_Init */


/* Comments ----------------------------------------------------------- */

/* In OSF/1 the functions signal(), sigvec() and sigblock() are obsolete.
   */
